import React, { Component } from 'react';
import * as d3 from "d3";
import PropTypes from 'prop-types';

class ArcProcess extends Component {
    constructor(props) {
        super(props);
        console.log(d3)
        this.arcRef= React.createRef();
    }

    componentDidMount() {
        let context =  this.setContext();
        this.$context = context;
        this.draw(context);
    }

    componentDidUpdate() {
        let context = this.$context;
        this.draw(context);
        // this.setText(context);
        // this.setUpperground(context);
    }

    draw(context) {
        this.setText(context);
        this.setBackground(context);
        this.setUpperground(context);
    }

    setContext() {
        const { width, height } = this.props;
        return d3.select(this.arcRef.current)
            .append('svg')
                .attr('width', width)
                .attr('height', height)
                .attr('viewBox', [0,0,width,height])
            .append('g')
                .attr('transform', `translate(${width/2}, ${height/2})`)
    }

    setBackground(context) {
        const { backgroundColor } = this.props;
        return context.selectAll('.arc-process-bg')
            .data([{ endAngle: Math.PI * 2 }])
            .join('path')
            .classed('arc-process-bg', true)
            .attr('fill', backgroundColor)
            .attr('d', this.arc())
    }

    setUpperground(context) {
        const { data, duration } = this.props;
        let percent = data / 100;
        let that = this;
        
        return context.selectAll('.arc-process')
            .data([{ endAngle: 0  }])
            .join('path')
            .classed('arc-process', true)
            // .attr('fill', this.color(data))
            .attr('d', this.arc())
            .transition()
                .ease(d3.easeLinear)
                .duration(duration)
                .attrTween('d',function(d) {
                    let compute = d3.interpolate(0, Math.PI * 2 * percent);
                    return function(t){
                        d.endAngle = compute(t);
                        //将新参数传入，生成新的圆弧构造器
                        return that.arc()(d);
                    }
                })
                .styleTween('fill',function(d){
                    return function(t){
                        var data = d.endAngle / Math.PI / 2 * 100;
                        //返回数值对应的色值
                        return that.color(data);
                    }
                })
    }

    setText(context) {
        const { data, decimal, duration, fontColor, fontSize } = this.props;
        return context.selectAll('.arc-process-text')
            .data([data])
            .join('text')
            .classed('arc-process-text', true)
            .attr('text-anchor','middle')
            .attr('dominant-baseline','middle')
            .attr('font-size', fontSize)
            .attr('stroke', fontColor)
            .transition()
            .duration(duration)
            .textTween(function(d) {
                let compute = d3.interpolate(0, d);
                let f = d3.format(`.${decimal}f`);
                return  function(t) { return f(compute(t)) + "%" };
            })
    }

    arc() {
        const { width, height, useCorner, innerRadius, outerRadius } = this.props;
        let size = d3.min([width, height]) / 2;
        let inner = this.toPoint(innerRadius) * size,
            outer = this.toPoint(outerRadius) * size,
            corner = useCorner ? (outer - inner) / 2 : 0;
        return d3.arc()
            .innerRadius(inner)
            .outerRadius(outer)
            .startAngle(0)
            .cornerRadius(() => corner)
    }

    color(data) {
        const { colors } = this.props;
        // console.log(colors)
        if(colors.length < 2) return colors[0];
        let color =  d3.scaleLinear().domain([0,100]).range(colors);
        // console.log(data)
        return color(data);
    }

    toPoint(percent) {
        return percent.replace('%', '') / 100;
    }
    
    render() {
        return <div ref={this.arcRef}></div>
    }
}

ArcProcess.defaultProps = {
    width: 300,
    height: 300,
    innerRadius: '60%',
    outerRadius: '80%',
    useCorner: false,
    data: 100,
    // backgroundColor: '#FDF5E6',
    backgroundColor: '#eeeeee',
    colors: ['#0997F7'],
    decimal: 0,
    duration: 1000,
    fontColor: '#000',
    fontSize: 26
}

ArcProcess.propTypes = {
    width: PropTypes.number,
    height: PropTypes.number,
    innerRadius: PropTypes.string,
    outerRadius: PropTypes.string,
    useCorner: PropTypes.bool,
    data: PropTypes.number.isRequired,
    backgroundColor: PropTypes.string,
    colors: PropTypes.array,
    decimal: PropTypes.number,
    duration: PropTypes.number,
    fontColor: PropTypes.string,
    fontSize: PropTypes.number,
};

export default ArcProcess;
